/***************************************************************************
                    bbcodegenarator.cpp  -  description
                             -------------------
    begin                : Jul 21 2009
    copyright            : (C) 2004-2009 by Andre Simon
    email                : andre.simon1@gmx.de
 ***************************************************************************/


/*
This file is part of Highlight.

Highlight is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Highlight is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Highlight.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <sstream>

#include "bbcodegenerator.h"

using namespace std;

namespace highlight
{

	string  BBCodeGenerator::getOpenTag (const ElementStyle & elem )
	{
		ostringstream s;

		s << "[color=#";
		s  << elem.getColour().getRed ( HTML )
		   << elem.getColour().getGreen ( HTML )
		   << elem.getColour().getBlue ( HTML )
		   << "]";

		if ( elem.isBold() ) s << "[b]";
		if ( elem.isItalic() ) s << "[i]";
		if ( elem.isUnderline() ) s << "[u]";
		return  s.str();
	}

	string  BBCodeGenerator::getCloseTag ( const ElementStyle &elem )
	{
		ostringstream s;
		if ( elem.isBold() ) s << "[/b]";
		if ( elem.isItalic() ) s << "[/i]";
		if ( elem.isUnderline() ) s << "[/u]";
		s << "[/color]";
		return  s.str();
	}


	bool BBCodeGenerator::initTheme ( const string& themePath )
	{
		bool result = CodeGenerator::initTheme ( themePath );
		styleTagOpen.push_back ( "");
		styleTagOpen.push_back ( getOpenTag ( docStyle.getStringStyle() ) );
		styleTagOpen.push_back ( getOpenTag ( docStyle.getNumberStyle() ) );
		styleTagOpen.push_back ( getOpenTag ( docStyle.getSingleLineCommentStyle() ) );
		styleTagOpen.push_back ( getOpenTag ( docStyle.getCommentStyle() ) );
		styleTagOpen.push_back ( getOpenTag ( docStyle.getEscapeCharStyle() ) );
		styleTagOpen.push_back ( getOpenTag ( docStyle.getDirectiveStyle() ) );
		styleTagOpen.push_back ( getOpenTag ( docStyle.getDirectiveStringStyle() ) );
		styleTagOpen.push_back ( getOpenTag ( docStyle.getLineStyle() ) );
		styleTagOpen.push_back ( getOpenTag ( docStyle.getSymbolStyle() ) );

		styleTagClose.push_back ( "" );
		styleTagClose.push_back ( getCloseTag ( docStyle.getStringStyle() ) );
		styleTagClose.push_back ( getCloseTag ( docStyle.getNumberStyle() ) );
		styleTagClose.push_back ( getCloseTag ( docStyle.getSingleLineCommentStyle() ) );
		styleTagClose.push_back ( getCloseTag ( docStyle.getCommentStyle() ) );
		styleTagClose.push_back ( getCloseTag ( docStyle.getEscapeCharStyle() ) );
		styleTagClose.push_back ( getCloseTag ( docStyle.getDirectiveStyle() ) );
		styleTagClose.push_back ( getCloseTag ( docStyle.getDirectiveStringStyle() ) );
		styleTagClose.push_back ( getCloseTag ( docStyle.getLineStyle() ) );
		styleTagClose.push_back ( getCloseTag ( docStyle.getSymbolStyle() ) );

		return result;
	}


	BBCodeGenerator::BBCodeGenerator() : CodeGenerator ( BBCODE )
	{
		newLineTag = "\n";
		spacer = " ";
	}

	BBCodeGenerator::~BBCodeGenerator() {}

	string BBCodeGenerator::getHeader()
	{
		return string();
	}

	void BBCodeGenerator::printBody()
	{
		*out << "[size="<<getBaseFontSize()<<"]";
		processRootState();
		*out << "[/size]"; 
	}

	string BBCodeGenerator::getFooter()
	{
		return string();
	}

	string BBCodeGenerator::maskCharacter ( unsigned char c )
	{
		return string ( 1, c );
	}

	string BBCodeGenerator::getMatchingOpenTag ( unsigned int styleID )
	{
		return getOpenTag (docStyle.getKeywordStyle ( langInfo.getKeywordClasses() [styleID] ) );
	}

	string BBCodeGenerator::getMatchingCloseTag ( unsigned int styleID )
	{
		return getCloseTag ( docStyle.getKeywordStyle ( langInfo.getKeywordClasses() [styleID] ) );
	}

}
